/* Highly inspirated by nehe lesson 4 port to GX by WinterMute and
   http://tehwii.tehskeen.com/tehwii/examples.php. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <gccore.h>
#include <wiiuse/wpad.h>


#define DEFAULT_FIFO_SIZE (256 * 1024)

/* Pointers to the allocated framebuffer memory areas. */
static u32 *frameBuffer = NULL ;
GXRModeObj *rmode ;




/****************************************************************************
* CvtRGB
*
* This function simply returns two RGB pixels as one Y1CbY2Cr.
  Thanks to: http://tehwii.tehskeen.com/tehwii/examples.php !!!
*****************************************************************************/
u32
CvtRGB (u8 r1, u8 g1, u8 b1, u8 r2, u8 g2, u8 b2)
{
  int y1, cb1, cr1, y2, cb2, cr2, cb, cr;

  y1 = (299 * r1 + 587 * g1 + 114 * b1) / 1000;
  cb1 = (-16874 * r1 - 33126 * g1 + 50000 * b1 + 12800000) / 100000;
  cr1 = (50000 * r1 - 41869 * g1 - 8131 * b1 + 12800000) / 100000;

  y2 = (299 * r2 + 587 * g2 + 114 * b2) / 1000;
  cb2 = (-16874 * r2 - 33126 * g2 + 50000 * b2 + 12800000) / 100000;
  cr2 = (50000 * r2 - 41869 * g2 - 8131 * b2 + 12800000) / 100000;

  cb = (cb1 + cb2) >> 1;
  cr = (cr1 + cr2) >> 1;

  return (y1 << 24) | (cb << 16) | (y2 << 8) | cr;
}



void draw_dot_at (int x, int y)
{
  if ((x < 0) || (x > rmode->fbWidth)) return ;
  if ((y < 0) || (y > rmode->xfbHeight)) return ;

  /* Yep, we must write 2 horizontal pixels at once since tha frame buffer is
     32 bits wide and each pixel is 16 bits. So don't forget to divide by 2
     while computing the index in the framebuffer's "array" from the x and y
     coordinates. */
  frameBuffer[y * (rmode->fbWidth / 2) + (x / 2)] =
    CvtRGB (0xFF, 0x00, 0x00, 0x00, 0xFF, 0x00) ;
}



int main (int argc, char **argv)
{
  GXColor background = {0, 0, 0, 0xff} ;
  void *gp_fifo = NULL;
  struct ir_t wiimote0_ir ;

  /* Initialize the Wii. */
  VIDEO_Init () ;
  WPAD_Init () ;

  /* Say we want to get buttons, acceleromater and IR-tracking info. */
  WPAD_SetDataFormat (WPAD_CHAN_0, WPAD_FMT_BTNS_ACC_IR) ;

  rmode = VIDEO_GetPreferredMode (NULL) ;

  /* Allocate 1 framebuffer. */
  frameBuffer = MEM_K0_TO_K1 (SYS_AllocateFramebuffer (rmode)) ;

  VIDEO_Configure(rmode) ;

  /* Switch to initial framebuffer. */
  VIDEO_SetNextFramebuffer (frameBuffer) ;
  VIDEO_SetBlack (FALSE) ;                   /* Make the display visible. */
  VIDEO_Flush () ;
  VIDEO_WaitVSync () ;                /* Wait for the next vertical retrace. */
  if (rmode->viTVMode & VI_NON_INTERLACE) VIDEO_WaitVSync () ;

  /* Setup the fifo and then init the flipper. */
  gp_fifo = memalign (32, DEFAULT_FIFO_SIZE) ;
  memset (gp_fifo, 0, DEFAULT_FIFO_SIZE) ;
  GX_Init (gp_fifo, DEFAULT_FIFO_SIZE) ;

  /* Clears the bg to color and clears the z buffer. */
  GX_SetCopyClear (background, 0x00ffffff) ;

  /* Copy the EFB to XFB. Flag tells if the color and Z buffers will */
  /* be cleared during the copy using values of GX_SetCopyClear).    */
  GX_CopyDisp (frameBuffer, GX_TRUE) ;

  for (;;) {
    /* Ask info about the position of the wiimote. */
    WPAD_IR (WPAD_CHAN_0, &wiimote0_ir) ;

    /* Do this before drawing. */
    GX_SetViewport (0, 0, rmode->fbWidth, rmode->efbHeight, 0, 1) ;

    draw_dot_at ((int) wiimote0_ir.x, (int) wiimote0_ir.y) ;

    /* Do this stuff after drawing. */
    GX_DrawDone () ;
    VIDEO_Flush () ;
    VIDEO_WaitVSync () ;

    /* Exit if requested. */
    WPAD_ScanPads () ;
    if (WPAD_ButtonsDown (WPAD_CHAN_0) & WPAD_BUTTON_HOME) exit(0) ;
    }

  return (0) ;
}
